package geniusweb.profile.utilityspace;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.DiscreteValueSet;
import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.NumberValueSet;
import geniusweb.issuevalue.ValueSet;
import tudelft.utilities.immutablelist.Range;
import tudelft.utilities.junit.GeneralTests;

public class NumberValueSetUtilTest
		extends GeneralTests<NumberValueSetUtilities> {

	private static final BigDecimal lowval1 = new BigDecimal("12.5");
	private static final BigDecimal low1util = new BigDecimal("0.5");
	private static final BigDecimal low1utilb = new BigDecimal("0.6");
	private static final BigDecimal highval = new BigDecimal("18.5");
	private static final BigDecimal highvalb = new BigDecimal("18.4");
	private static final BigDecimal highutil = new BigDecimal("0.9");
	private static final NumberValueSetUtilities valueset1 = new NumberValueSetUtilities(
			lowval1, low1util, highval, highutil);
	private static final NumberValueSetUtilities valueset1b = new NumberValueSetUtilities(
			lowval1, low1util, highval, highutil);
	private static final NumberValueSetUtilities valueset2 = new NumberValueSetUtilities(
			lowval1, low1utilb, highval, highutil);
	private static final NumberValueSetUtilities valueset3 = new NumberValueSetUtilities(
			lowval1, low1util, highvalb, highutil);
	private static final BigDecimal ZERO_1 = new BigDecimal("0.1");
	private final String utilstring = "{\"NumberValueSetUtilities\":{\"lowValue\":12.5,\"lowUtility\":0.5,\"highValue\":18.5,\"highUtility\":0.9}}";

	private static final ObjectMapper jackson = new ObjectMapper();

	@Override
	public List<List<NumberValueSetUtilities>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(valueset1, valueset1b),
				Arrays.asList(valueset2), Arrays.asList(valueset3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("NumberValueSetUtilities\\(12.5->0.5,18.5->0.9\\)",
				"NumberValueSetUtilities\\(12.5->0.6,18.5->0.9\\)",
				"NumberValueSetUtilities\\(12.5->0.5,18.4->0.9\\)");
	}

	@Test
	public void smoketest() {
		NumberValueSetUtilities utils = valueset1;
	}

	@Test
	public void utilAscentingTest() {
		NumberValueSetUtilities utils = valueset1;
		assertEquals(0,
				low1util.compareTo(utils.getUtility(new NumberValue(lowval1))));
	}

	@Test
	public void utilDescendingTest() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				highutil, highval, low1util);
		assertEquals(0,
				highutil.compareTo(utils.getUtility(new NumberValue(lowval1))));
	}

	@Test(expected = IllegalArgumentException.class)
	public void reverserangetest() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(highval,
				low1util, highval, highutil);
	}

	@Test(expected = NullPointerException.class)
	public void testNullHigh() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				low1util, null, highutil);
	}

	@Test(expected = NullPointerException.class)
	public void testNullHighUtil() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				low1util, highval, null);
	}

	@Test(expected = NullPointerException.class)
	public void testNullLow() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(null,
				low1util, highval, highutil);
	}

	@Test(expected = NullPointerException.class)
	public void testNullLowUtil() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				null, highval, highutil);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testLowUtilNegative() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				new BigDecimal("-0.2"), highval, highutil);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testHighUtilNegative() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				low1util, highval, new BigDecimal("-0.000003"));
	}

	@Test
	public void getUtilTest() {
		NumberValueSetUtilities utils = new NumberValueSetUtilities(lowval1,
				low1util, highval, highutil);
		assertEquals(0,
				highutil.compareTo(utils.getUtility(new NumberValue(highval))));
		assertEquals(0,
				low1util.compareTo(utils.getUtility(new NumberValue(lowval1))));
		// linear interpolated , check center
		BigDecimal middleutil = low1util.add(highutil)
				.divide(new BigDecimal("2"));
		NumberValue middleval = new NumberValue(
				lowval1.add(highval).divide(new BigDecimal("2")));
		assertEquals(0, middleutil.compareTo(utils.getUtility(middleval)));

	}

	@Test
	public void testIsNotFittingRange() {
		ValueSet valueset = new NumberValueSet(
				new Range(low1util, highutil, new BigDecimal("0.02")));
		ValueSetUtilities utilset = new NumberValueSetUtilities(lowval1,
				low1util, highval, highutil);
		assertEquals(
				"the utilities are specified down to 12.5 but the valueset starts at 0.5",
				utilset.isFitting(valueset));
	}

	@Test
	public void testTopOfRangeTooLarge() {
		ValueSet valueset = new NumberValueSet(
				new Range(lowval1, highval.subtract(ZERO_1), ZERO_1));
		ValueSetUtilities utilset = new NumberValueSetUtilities(lowval1,
				low1util, highval, highutil);
		assertEquals(
				"the utilities are specified up to 18.5 but the valueset ends at 18.4",
				utilset.isFitting(valueset));
	}

	@Test
	public void testIsFitting() {
		ValueSet valueset = new NumberValueSet(
				new Range(lowval1, highval, BigDecimal.ONE));
		ValueSetUtilities utilset = new NumberValueSetUtilities(lowval1,
				low1util, highval, highutil);
		assertEquals(null, utilset.isFitting(valueset));
	}

	@Test
	public void testIsFittingWrongType() {
		DiscreteValue vala = new DiscreteValue("a");
		DiscreteValue valb = new DiscreteValue("b");
		ValueSet valueset = new DiscreteValueSet(Arrays.asList(vala, valb));

		NumberValueSetUtilities utilset = new NumberValueSetUtilities(lowval1,
				low1util, highval, highutil);
		assertEquals(
				"The utilities are for a number valueset but the given values are DiscreteValueSet[\"a\", \"b\"]",
				utilset.isFitting(valueset));

	}

	@Test
	public void testSerialize() throws JsonProcessingException {

		String json = jackson.writeValueAsString(valueset1);
		System.out.println(json);
		assertEquals(utilstring, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ValueSetUtilities valueutils = jackson.readValue(utilstring,
				ValueSetUtilities.class);
		assertEquals(valueset1, valueutils);

	}

}
